import configparser
import os
from os.path import dirname

from ..logs_manager import Logger as log
from ..utils import close
from pathlib import Path

MCSNIPERPY_BACKEND_DIR = os.path.join(str(Path.home()), '.mcsniperpy')
MCSNIPERPY_CONFIG_FILE_PATH = os.path.join(MCSNIPERPY_BACKEND_DIR, 'backend_config.ini')

DEFAULT_ACCOUNTS_FILE = """Clear this file and write accounts in this format
email:pass:answer:answer:answer

or

email:pass

you can separate those accounts by a new line if you would like to use multiple accounts.

# You can comment line out by making them start with a #"""


class BackConfig:
    """Backend configuration for MCsniperPY

    The main purpose of this is to store the path for the \"frontend\" directory.
    The frontend directory stores the config file and accounts file."""

    def __init__(self):

        self.config = configparser.ConfigParser()

        if not os.path.isdir(MCSNIPERPY_BACKEND_DIR):
            os.mkdir(MCSNIPERPY_BACKEND_DIR)

        if os.path.isfile(MCSNIPERPY_CONFIG_FILE_PATH):
            self.config.read(MCSNIPERPY_CONFIG_FILE_PATH)
        else:
            log.error("Failed to find MCsniperPY directory")
            log.info("To fix this open your sniping folder (or make one) and type `mcsniperpy init` in it.")
            log.info("Next, just re-run the sniper with the command you used to get here.")
            close(1)

        if not self.validate_sections():
            log.error('invalid config file. please make sure it looks like:\n[sniper]\ninit_path =')
            close(1)

    def validate_sections(self):
        necessary_sections = ['sniper']
        sections = self.config.sections()
        log.debug(sections)
        return sections == necessary_sections


class Config:
    def __init__(self, config_path):
        log.debug(f'config path: {config_path}')
        user_config = configparser.ConfigParser(allow_no_value=True)
        user_config.read(config_path)

        self.config = user_config


def create_user_config() -> configparser.ConfigParser:
    user_config = configparser.ConfigParser(allow_no_value=True)
    user_config['sniper'] = {
        'timing_system': 'kqzz_api',
        'auto_claim_namemc': 'no',
        'snipe_requests': '3',
    }

    user_config['accounts'] = {
        'max_accounts': '30',
        'authentication_delay': '500',
        'start_authentication': '720'
    }

    user_config['skin'] = {
        'change_skin_on_snipe': 'no',
        'skin_change_type': 'url',
        'skin': ''
    }

    user_config.set('skin',
                    '; skin_change_type can be url, path, or username. refer to docs for more info.')

    return user_config


def populate_configs():
    if not log.yes_or_no('Are you sure you want to initialize your config in your cwd:'):
        close(1)

    config = configparser.ConfigParser()
    config['sniper'] = {"init_path": os.getcwd()}

    if not os.path.isdir(MCSNIPERPY_BACKEND_DIR):
        os.mkdir(MCSNIPERPY_BACKEND_DIR)

    with open(MCSNIPERPY_CONFIG_FILE_PATH, "w") as f:
        config.write(f)

    accounts_path = os.path.join(config['sniper']['init_path'], "accounts.txt")
    user_config_path = os.path.join(config['sniper']['init_path'], "config.ini")

    if os.path.isfile(accounts_path):
        if log.yes_or_no('Overwrite current accounts file:'):
            with open(os.path.join(config['sniper']['init_path'], "accounts.txt"), "w") as f:
                f.write(DEFAULT_ACCOUNTS_FILE)
    else:
        with open(os.path.join(config['sniper']['init_path'], "accounts.txt"), "w") as f:
            f.write(DEFAULT_ACCOUNTS_FILE)

    if os.path.isfile(user_config_path):
        if log.yes_or_no('Overwrite current config file:'):
            with open(os.path.join(config['sniper']['init_path'], "config.ini"), "w") as f:
                create_user_config().write(f)
    else:
        with open(os.path.join(config['sniper']['init_path'], "config.ini"), "w") as f:
            create_user_config().write(f)

    log.info("successfully initialized sniper")



import asyncio
import shutil

import typer

from mcsniperpy import Sniper
from mcsniperpy.util import logs_manager as log
from mcsniperpy.util import ping_tester
from mcsniperpy.util.name_system import next_name

app = typer.Typer()


def startup():
    width = shutil.get_terminal_size().columns
    print(f"""{log.Color.cyan}
    ███╗   ███╗ ██████╗███████╗███╗   ██╗██╗██████╗ ███████╗██████╗ ██████╗ ██╗   ██╗
    ████╗ ████║██╔════╝██╔════╝████╗  ██║██║██╔══██╗██╔════╝██╔══██╗██╔══██╗╚██╗ ██╔╝
    ██╔████╔██║██║     ███████╗██╔██╗ ██║██║██████╔╝█████╗  ██████╔╝██████╔╝ ╚████╔╝ 
    ██║╚██╔╝██║██║     ╚════██║██║╚██╗██║██║██╔═══╝ ██╔══╝  ██╔══██╗██╔═══╝   ╚██╔╝  
    ██║ ╚═╝ ██║╚██████╗███████║██║ ╚████║██║██║     ███████╗██║  ██║██║        ██║   
    ╚═╝     ╚═╝ ╚═════╝╚══════╝╚═╝  ╚═══╝╚═╝╚═╝     ╚══════╝╚═╝  ╚═╝╚═╝        ╚═╝   
    """.center(width))
    print(f"{log.Color.cyan}Created by Kqzz#0001")
    print(f"Git: github.com/MCSniperPY | Discord: https://mcsniperpy.github.io/discord{log.Color.white}")


sniper = Sniper(
    log.Color,
    log.Logger
)


@app.command()
def snipe(username: str = typer.Option(None),
          offset: int = typer.Option(None),
          debug: bool = typer.Option(False),
          color: bool = typer.Option(True),
          next_sorry: int = typer.Option(None, '--next')):
    if debug:
        sniper.log.debug_enabled = True

    if not color:
        sniper.color.disable()

    if next_sorry is not None:
        username = next_name(searches=next_sorry)

    startup()

    asyncio.get_event_loop().run_until_complete(sniper.run(username, offset))
    sniper.on_shutdown()


@app.command()
def ping(iterations: int = typer.Option(5)):
    asyncio.get_event_loop().run_until_complete(ping_tester.ping_test(iterations))
    sniper.on_shutdown()


@app.command()
def init():
    sniper.init()


def cli():
    try:
        app()
    except Exception as ex:
        tb = ex.__traceback__
        sniper.log.error(f"type: {type(ex).__name__}")
        sniper.log.error(f"message: {str(ex)}")
        while tb is not None:
            sniper.log.error(f"{tb.tb_frame.f_code.co_filename}:{tb.tb_lineno}")
            tb = tb.tb_next
    finally:
        sniper.on_shutdown()


if __name__ == '__name__':
    cli()
